package Role::Parent;
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use t::lib::MooseCompat;
use Mouse::Role;

sub meth2  { }
sub meth1 { }

1;
